@@ -2,10 +2,10 @@
 #
 # The PyMca X-Ray Fluorescence Toolkit
 #
-# Copyright (c) 2004-2020 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2022 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMca X-ray Fluorescence Toolkit developed at
-# the ESRF by the Software group.
+# the ESRF.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -26,7 +26,7 @@
 # THE SOFTWARE.
 #
 #############################################################################*/
-__author__ = "V.A. Sole - ESRF Data Analysis"
+__author__ = "V.A. Sole - ESRF"
 __contact__ = "sole@esrf.fr"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
@@ -83,6 +83,8 @@ class EDFStack(DataObject.DataObject):
         self.sourceType = SOURCE_TYPE
         self.info = {}
         self.nbFiles=len(filelist)
+        fileSampling = 1
+        mcaSampling = 1
 
         #read first edf file
         #get information
@@ -364,47 +366,68 @@ class EDFStack(DataObject.DataObject):
                                                    arrRet.shape[1]),
                                                    self.__dtype)
                         except:
-                            try:
-                                needed_ = self.nbFiles * \
-                                           arrRet.shape[0] *\
-                                           arrRet.shape[1] * 4
-                                physicalMemory = PhysicalMemory.getPhysicalMemoryOrNone()
-                                if physicalMemory is not None:
-                                    # spare 5 % of memory
-                                    if physicalMemory < (1.05 * needed_):
-                                        raise MemoryError("Not enough physical memory available")
-                                self.data = numpy.zeros((self.nbFiles,
-                                                   arrRet.shape[0],
-                                                   arrRet.shape[1]),
-                                                   numpy.float32)
-                            except (MemoryError, ValueError):
-                                text = "Memory Error: Attempt subsampling or convert to HDF5"
-                                if HDF5 and (('PyMcaQt' in sys.modules) or\
-                                   ('PyMca.PyMcaQt' in sys.modules)):
-                                    from PyMca5 import PyMcaQt as qt
-                                    from PyMca5 import ArraySave
+                            text = "Memory Error: Attempt subsampling or convert to HDF5"
+                            if 1:
+                                if 'PyMca5.PyMcaGui.PyMcaQt' in sys.modules:
+                                    from PyMca5.PyMcaGui import PyMcaQt as qt
                                     msg=qt.QMessageBox.information( None,
                                       "Memory error\n",
-                                      "Do you want to convert your data to HDF5?\n",
-                                      qt.QMessageBox.Yes,qt.QMessageBox.No)
+                                      "Do you want to subsample your data?\n",
+                                    qt.QMessageBox.Yes,qt.QMessageBox.No)
                                     if msg == qt.QMessageBox.No:
                                         raise MemoryError(text)
-                                    hdf5file = qt.QFileDialog.getSaveFileName(None,
-                                                "Please select output file name",
-                                                os.path.dirname(filelist[0]),
-                                                "HDF5 files *.h5")
-                                    if not len(hdf5file):
-                                        raise IOError(\
-                                            "Invalid output file")
-                                    hdf5file = qt.safe_str(hdf5file)
-                                    if not hdf5file.endswith(".h5"):
-                                        hdf5file += ".h5"
-                                    hdf, self.data =  ArraySave.getHDF5FileInstanceAndBuffer(hdf5file,
-                                                  (self.nbFiles,
-                                                   arrRet.shape[0],
-                                                   arrRet.shape[1]))
-                                else:
-                                    raise MemoryError("Memory Error")
+                                self.data = None
+                                self.__dtype = numpy.float32
+                                nTry = 0
+                                while self.data is None:
+                                    try:
+                                        self.data = numpy.zeros((len(numpy.arange(self.nbFiles)[::fileSampling]),
+                                                           len(numpy.arange(arrRet.shape[0])[::mcaSampling]),
+                                                           arrRet.shape[1]),
+                                                           self.__dtype)
+                                        _logger.warning("Subsampling data by %d x %d " % (fileSampling, mcaSampling))
+                                        _logger.warning("Data shape %d x %d x %d " % (self.data.shape[0],
+                                                                                      self.data.shape[1],
+                                                                                      self.data.shape[2]))
+                                    except:
+                                        if 10 * fileSampling < self.nbFiles:
+                                            fileSampling += 1
+                                        if 10 * mcaSampling < arrRet.shape[0]:
+                                            mcaSampling += 1
+                                        nTry += 1
+                                        if nTry > 100:
+                                            raise MemoryError("Memory Error and I could not subsample")
+                            elif HDF5 and ('PyMca5.PyMcaGui.PyMcaQt' in sys.modules):
+                                from PyMca5.PyMcaGui import PyMcaQt as qt
+                                from PyMca5.PyMcaGui.io import PyMcaFileDialogs
+                                from PyMca5.PyMcaIO import ArraySave
+                                msg=qt.QMessageBox.information( None,
+                                  "Memory error\n",
+                                  "Do you want to convert your data to HDF5?\n",
+                                  qt.QMessageBox.Yes,qt.QMessageBox.No)
+                                if msg == qt.QMessageBox.No:
+                                    raise MemoryError(text)
+                                hdf5file = PyMcaFileDialogs.getFileList(parent=None,
+                                                        message="Please select output file name",
+                                                        currentdir=os.path.dirname(filelist[0]),
+                                                        mode="SAVE",
+                                                        single=True,
+                                                        filetypelist=["HDF5 files *.h5"])
+                                if not len(hdf5file):
+                                    raise IOError(\
+                                        "Invalid output file")
+                                hdf5file = qt.safe_str(hdf5file[0])
+                                if not hdf5file.endswith(".h5"):
+                                    hdf5file += ".h5"
+                                hdf, self.data =  ArraySave.getHDF5FileInstanceAndBuffer(hdf5file,
+                                              (self.nbFiles,
+                                               arrRet.shape[0],
+                                               arrRet.shape[1]))
+                            else:
+                                raise MemoryError("Memory Error")
+                    filelist = filelist[::fileSampling]
+                    self.sourceName = self.sourceName[::fileSampling]
+                    self.nbFiles = len(filelist)
                     self.incrProgressBar=0
                     if fileindex == 1:
                         for tempEdfFileName in filelist:
@@ -412,7 +435,7 @@ class EDFStack(DataObject.DataObject):
                             pieceOfStack=tempEdf.GetData(0)
                             self.data[:,self.incrProgressBar,:] = pieceOfStack[:,:]
                             self.incrProgressBar += 1
-                            self.onProgress(self.incrProgressBar)
+                            self.onProgress(self.incrProgressBar * fileSampling)
                     else:
                         # test for ID24 map
                         ID24 = False
@@ -450,7 +473,7 @@ class EDFStack(DataObject.DataObject):
                             else:
                                 pieceOfStack=tempEdf.GetData(0)
                             try:
-                                self.data[self.incrProgressBar, :,:] = pieceOfStack[:,:]
+                                self.data[self.incrProgressBar, :,:] = pieceOfStack[::mcaSampling,:]
                             except:
                                 if pieceOfStack.shape[1] != arrRet.shape[1]:
                                     _logger.warning(" ERROR on file %s", tempEdfFileName)
@@ -462,9 +485,9 @@ class EDFStack(DataObject.DataObject):
                                           :pieceOfStack.shape[0],
                                           :pieceOfStack.shape[1]] = pieceOfStack[:, :]
                             self.incrProgressBar += 1
-                            self.onProgress(self.incrProgressBar)
+                            self.onProgress(self.incrProgressBar * fileSampling)
                     self.onEnd()
-        self.__nFiles         = self.incrProgressBar
+        self.__nFiles = self.incrProgressBar
         self.__nImagesPerFile = nImages
         shape = self.data.shape
         for i in range(len(shape)):
